﻿using Hotline.Share.Enums.Order;
using Hotline.Share.Requests;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Snapshot;

/// <summary>
/// 随手拍统计输出Dto
/// </summary>
public class SnapshotStatisticsOutDto
{
    /// <summary>
    /// 随手拍来件数
    /// </summary>
    public int SSPLJS => WZSLFWNJS + SSPZ12345JS + SLFWNZJS;

    /// <summary>
    /// 未在受理范围内件数
    /// </summary>
    public int WZSLFWNJS { get; set; }

    /// <summary>
    /// 随手拍转12345件数
    /// </summary>
    public int SSPZ12345JS { get; set; }

    /// <summary>
    /// 受理范围内总件数
    /// </summary>
    public int SLFWNZJS { get; set; }

    /// <summary>
    /// 受理范围内派给各网格员件数
    /// </summary>
    public int SLFWNPGGWGYJS => SLFWNPGWGYSXSNHFJS + SLFWNPGWGYCGSXSHFJS + SLFWNPGWGYWHFJS;

    /// <summary>
    /// 受理范围内派给网格员四小时内回复件数
    /// </summary>
    public int SLFWNPGWGYSXSNHFJS { get; set; }

    /// <summary>
    /// 受理范围内派给网格员超过四小时回复件数
    /// </summary>
    public int SLFWNPGWGYCGSXSHFJS { get; set; }

    /// <summary>
    /// 受理范围内派给网格员未回复件数
    /// </summary>
    public int SLFWNPGWGYWHFJS { get; set; }

    /// <summary>
    /// 受理范围内按12345直派给各区县或部门件数
    /// </summary>
    public int SLFWNA12345ZPGGQXHBMJS => SLFWNA12345ZPGGBMJS + SLFWNA12345ZPGGQXJS;

    /// <summary>
    /// 受理范围内按12345直派给各部门件数
    /// </summary>
    public int SLFWNA12345ZPGGBMJS { get; set; }

    /// <summary>
    /// 受理范围内按12345直派给各区县件数
    /// </summary>
    public int SLFWNA12345ZPGGQXJS { get; set; }

    /// <summary>
    /// 受理范围内已办信件
    /// </summary>
    public int SLFWNYBXJ => ZXYB + BMYB;

    /// <summary>
    /// 中心已办
    /// </summary>
    public int ZXYB { get; set; }

    /// <summary>
    /// 部门已办
    /// </summary>
    public int BMYB { get; set; }

    /// <summary>
    /// 受理范围满意度
    /// </summary>
    public double SLFWMYD { get; set; }

    /// <summary>
    /// 受理范围满意度
    /// </summary>
    public string SLFWMYDTxt => SLFWMYD.ToString("f4") + "%";

    /// <summary>
    /// 满意量
    /// </summary>
    public int MYL { get; set; }

    /// <summary>
    /// 不满意量
    /// </summary>
    public int BMYL { get; set; }

    /// <summary>
    /// 审核发放奖励个数
    /// </summary>
    public int SHFFJLGS => SPBTYHBGS + SPTYHBGS;

    /// <summary>
    /// 审批不同意红包个数
    /// </summary>
    public int SPBTYHBGS { get; set; }

    /// <summary>
    /// 审批同意红包个数
    /// </summary>
    public int SPTYHBGS { get; set; }

    /// <summary>
    /// 特殊红包审批统计
    /// </summary>
    public int TSHBSP { get; set; }

    /// <summary>
    /// 审批同意个数
    /// </summary>
    public int SPTYGS { get; set; }

    /// <summary>
    /// 已发金额
    /// </summary>
    public double? YFJE { get; set; }

    /// <summary>
    /// 局审核发放网格员奖励个数
    /// </summary>
    public int JSHFFWGJLGS { get; set; }

    /// <summary>
    /// 审批同意（网格员）红包个数
    /// </summary>
    public int SPTYWGYHBGS { get; set; }

    /// <summary>
    /// 审批不同意（网格员）红包个数
    /// </summary>
    public int SPBTYWGYHBGS { get; set; }

    /// <summary>
    /// 市民奖励总额
    /// </summary>
    public double? SMJLZE { get; set; }

    /// <summary>
    /// 市民已发放奖励总额
    /// </summary>
    public double? SMYFFJLZE { get; set; }

    /// <summary>
    /// 市民待发奖励总额
    /// </summary>
    public double? SMDFFJLZE { get; set; }

    /// <summary>
    /// 已发（个）
    /// </summary>
    public int YFG { get; set; }

    /// <summary>
    /// 无法联系（个）
    /// </summary>
    public int WFLXG { get; set; }

    /// <summary>
    /// 婉拒红包（个）
    /// </summary>
    public int WJHBG { get; set; }

    /// <summary>
    /// 网格员应发奖励金额
    /// </summary>
    public double? WGYYFJLJE { get; set; }

    /// <summary>
    /// 网格员已发放奖励总额
    /// </summary>
    public double? WGYYFFJLZE { get; set; }

    /// <summary>
    /// 网格员待发放奖励总额
    /// </summary>
    public double? WGYDFFJLZE { get; set; }

    /// <summary>
    /// 网格员扣款总额（已发）
    /// </summary>
    public float WGYKKZEYF { get; set; }

    /// <summary>
    /// 网格员扣款总额（待发）
    /// </summary>
    public float WGYKKZEDF { get; set; }

    /// <summary>
    /// 受理范围内多部门会签件件数
    /// </summary>
    public int SLFWNDBMHQJJS { get; set; }

    /// <summary>
    /// 受理范围内热线中心归档件数
    /// </summary>
    public int SLFWNRXZXGDJS { get; set; }

    /// <summary>
    /// 热线中心发起会签件件数
    /// </summary>
    public int RXZXFQHQJJS { get; set; }

    /// <summary>
    /// 安全隐患
    /// </summary>
    public int AQYH { get; set; }

    /// <summary>
    /// 已完成安全隐患整改
    /// </summary>
    public int YWCAQYHZG { get; set; }

    /// <summary>
    /// 申请延期工单件数
    /// </summary>
    public int SQYQGDJS { get; set; }

    /// <summary>
    /// 申请延期工单次数
    /// </summary>
    public int SQYQGDCS { get; set; }

    /// <summary>
    /// 超期件
    /// </summary>
    public int CQJ { get; set; }

    /// <summary>
    /// 二次办理件数
    /// </summary>
    public int ECBLJS => ECBLJSTHBM + ECBLJSHFBMYCB + ECBLJSTTDYYJBM;

    /// <summary>
    /// 二次办理件数-退回部门
    /// </summary>
    public int ECBLJSTHBM { get; set; }

    /// <summary>
    /// 二次办理件数-回访不满意重办
    /// </summary>
    public int ECBLJSHFBMYCB { get; set; }

    /// <summary>
    /// 二次办理件数-回访满意
    /// </summary>
    public int ECBLJSHFMY { get; set; }

    /// <summary>
    /// 二次办理件数-特提到原一级部门
    /// </summary>
    public int ECBLJSTTDYYJBM { get; set; }

    /// <summary>
    /// 二次办理工单满意率
    /// </summary>
    public double ECBLGDMYL => ECBLJS == 0 ? 0 : (ECBLJSHFMY / ECBLJS) * 100;

    /// <summary>
    /// 二次办理工单满意率
    /// </summary>
    public string ECBLGDMYLTxt => ECBLGDMYL.ToString("f4") + "%";

    /// <summary>
    /// 二次办理工单件数-退回部门回访满意
    /// </summary>
    public int ECBLGDJSTMBMHFMYD { get; set; }

    /// <summary>
    /// 二次办理工单满意率-退回部门
    /// </summary>
    public double ECBLGDMYLTHBM => ECBLJSTHBM == 0 ? 0 : (ECBLGDJSTMBMHFMYD / ECBLJSTHBM) * 100;

    /// <summary>
    /// 二次办理工单满意率-退回部门
    /// </summary>
    public string ECBLGDMYLTHBMTxt => ECBLGDMYLTHBM.ToString("f4") + "%";

    /// <summary>
    /// 二次办理工单满意率-回访不满意重办
    /// </summary>
    public double ECBLGDMYLHFBMYCB => ECBLJS == 0 ? 0 : (ECBLJSHFBMYCB / ECBLJS) * 100;

    /// <summary>
    /// 二次办理工单满意率-回访不满意重办
    /// </summary>
    public string ECBLGDMYLHFBMYCBTxt => ECBLGDMYLHFBMYCB.ToString("f4") + "%"; 

    /// <summary>
    /// 特提件数
    /// </summary>
    public int TTJS => TTDYYJBMJS + TTDPDZJS + QTTTJS;

    /// <summary>
    /// 特提到原一级部门件数
    /// </summary>
    public int TTDYYJBMJS { get; set; }

    /// <summary>
    /// 特提到派单组件数
    /// </summary>
    public int TTDPDZJS { get; set; }

    /// <summary>
    /// 其他特提件数
    /// </summary>
    public int QTTTJS { get; set; }
}

/// <summary>
/// 随手拍统计输入Dto
/// </summary>
public class SnapshotStatisticsInDto
{
    /// <summary>
    /// 开始时间
    /// </summary>
    [Required]
    public DateTime StartTime { get; set; }

    /// <summary>
    /// 结束时间
    /// </summary>
    [Required]
    public DateTime EndTime { get; set; }

    /// <summary>
    /// 行业Id
    /// </summary>
    public string? IndustryId { get; set; }
}

public record SnapshotStatisticsDetailInDto : PagedRequest
{
    /// <summary>
    /// 开始时间
    /// </summary>
    [Required]
    public DateTime StartTime { get; set; }

    /// <summary>
    /// 结束时间
    /// </summary>
    [Required]
    public DateTime EndTime { get; set; }

    /// <summary>
    /// 字段名称
    /// </summary>
    [Required]
    public string FieldName { get; set; }
}

public class SnapshotStatisticsDetailOutDto
{
    /// <summary>
    /// Id
    /// </summary>
    public string Id { get; set; }

    /// <summary>
    /// 过期状态
    /// </summary>
    public EExpiredStatus ExpiredStatus { get; set; }

    /// <summary>
    /// 过期状态
    /// </summary>
    public string ExpiredStatusText => ExpiredStatus.GetDescription();

    /// <summary>
    /// 受理编号
    /// </summary>
    public string No { get; set; }

    /// <summary>
    /// 信件状态
    /// </summary>
    public EOrderStatus Status { get; set; }

    /// <summary>
    /// 信件状态
    /// </summary>
    public string StatusTxt => Status.GetDescription();

    /// <summary>
    /// 来源
    /// </summary>
    public string SourceChannel { get; set; }

    /// <summary>
    /// 当前节点
    /// </summary>
    public string CurrentStepName { get; set; }

    /// <summary>
    /// 重办次数
    /// </summary>
    public int ReTransactNum { get; set; }

    /// <summary>
    /// 是否紧急
    /// </summary>
    public bool IsUrgent { get; set; }

    /// <summary>
    /// 是否紧急
    /// </summary>
    public string IsUrgentTxt => IsUrgent ? "紧急" : "-";

    /// <summary>
    /// 期满时间
    /// </summary>
    public DateTime? ExpiredTime { get; set; }

    /// <summary>
    /// 是否超期
    /// </summary>
    public bool IsExpired
    {
        get
        {
            if (ExpiredTime.HasValue)
                return DateTime.Now > ExpiredTime.Value;
            return false;
        }
    }

    /// <summary>
    /// 受理时间
    /// </summary>
    public DateTime CreationTime { get; set; }

    /// <summary>
    /// 标题
    /// </summary>
    public string Title { get; set; }

    /// <summary>
    /// 行业
    /// </summary>
    public string IndustryName { get; set; }

    /// <summary>
    /// 办理时长
    /// </summary>
    public double CreationTimeHandleDuration { get; set; }

    /// <summary>
    /// 区域
    /// </summary>
    public string County { get; set; }

    /// <summary>
    /// 电话
    /// </summary>
    public string Contact { get; set; }

    /// <summary>
    /// 网格员名称
    /// </summary>
    public string MemberName { get; set; }

    /// <summary>
    /// 网格员电话
    /// </summary>
    public string MemberMobile { get; set; }

    /// <summary>
    /// 网格员回复
    /// </summary>
    public string NetworkRemark { get; set; }

    /// <summary>
    /// 网格E通编号
    /// </summary>
    public string? NetworkENumber { get; set; }

    /// <summary>
    /// 受理期满时间
    /// </summary>
    public DateTime? AcceptExpiredTime { get; set; }

    /// <summary>
    /// 一级部门
    /// </summary>
    public string OrgLevelOneCode { get; set; }

    /// <summary>
    /// 接办部门
    /// </summary>
    public string ActualHandleOrgName { get; set; }

    /// <summary>
    /// 接办时间
    /// </summary>
    public DateTime? StartTime { get; set; }

    /// <summary>
    /// 办结时间
    /// </summary>
    public DateTime? CenterToOrgTime { get; set; }

    /// <summary>
    /// 受理类型
    /// </summary>
    public string AcceptType { get; set; }

    /// <summary>
    /// 热点类型
    /// </summary>
    public string HotspotName { get; set; }

    /// <summary>
    /// 受理人
    /// </summary>
    public string AcceptorName { get; set; }

    /// <summary>
    /// 受理内容
    /// </summary>
    public string Content { get; set; }

    /// <summary>
    /// 办理结果
    /// </summary>
    public string FileOpinion { get; set; }
}
public class StatisticsDto
{
}
